﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GOAPAction.h"
#include "Components/ActorComponent.h"
#include "Containers/UnrealString.h"
#include "WorldState.h"
#include "GOAPAgent.generated.h"

UCLASS()
class UActionNode : public UObject
{
	GENERATED_BODY()

public:
	UActionNode* Parent;
	UGOAPAction* Action;
	UWorldState* State;
	float Cost;

	void Init(UActionNode* ParentNode, UGOAPAction* GOAPAction, UWorldState* ProceduralState, float TotalCost)
	{
		Parent = ParentNode;
		Action = GOAPAction;
		State = ProceduralState;
		Cost = TotalCost;
	}
};

UCLASS(ClassGroup=(Custom), meta=(BlueprintSpawnableComponent))
class COMP250_1_2101327_AI_API UGOAPAgent : public UActorComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this component's properties
	UGOAPAgent();

	UPROPERTY(EditAnywhere)
	TArray<UGOAPAction*> AvailableActions;

	UPROPERTY()
	TMap<FString, int> Goals
	{
		{"PlayerHealth", 0}
	};

protected:
	// Called when the game starts
	virtual void BeginPlay() override;

	UFUNCTION()
	bool BuildActionGraph(UActionNode* Parent, TArray<UActionNode*>& SuccessfulLeaves, UWorldState* WorldState, TMap<FString, int> AgentGoals);

	TSet<UWorldState*> SeenStates;
public:
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

	UFUNCTION()
	TArray<UGOAPAction*> Plan(UWorldState* WorldState, TMap<FString, int> AgentGoals);
};
